@@ -12,7 +12,7 @@ class TestExamples(unittest.TestCase):
     def test_run_every_file(self):
         i = -1
         for i, file in enumerate(pathlib.Path(__file__).parent.parent.joinpath('examples').glob('*.py')):
-            if 'tcp' not in str(file) and 'udp' not in str(file):
+            if 'tcp' not in str(file) and 'udp' not in str(file) and 'live' not in str(file):
                 env = os.environ
                 env['PYTHONPATH'] = f':{pathlib.Path(__file__).parent.parent.absolute()}'
                 assert subprocess.check_call(f'python3 {file}'.split(), env=env, shell=False) == 0
@@ -22,4 +22,4 @@ class TestExamples(unittest.TestCase):
         if csv_file.exists():
             csv_file.unlink()
 
-        assert i == 11
+        assert i == 12
